@@ -1,5 +1,3 @@
-package AppConfig::State;
-
 #============================================================================
 #
 # AppConfig::State.pm
@@ -11,11 +9,9 @@ package AppConfig::State;
 #
 # Written by Andy Wardley <abw@wardley.org>
 #
-# Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
+# Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
 # Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 #
-# $Id: State.pm,v 1.61 2004/02/04 10:11:23 abw Exp $
-#
 #----------------------------------------------------------------------------
 #
 # TODO
@@ -40,18 +36,16 @@ package AppConfig::State;
 #
 #============================================================================
 
-require 5.004;
-
+package AppConfig::State;
 use strict;
+use warnings;
 
-use vars qw( $VERSION $DEBUG $AUTOLOAD );
-BEGIN {
-	$VERSION = '1.64';
-	$DEBUG   = 0;
-}
+our $VERSION = '1.67';
+our $DEBUG   = 0;
+our $AUTOLOAD;
 
 # need access to AppConfig::ARGCOUNT_*
-use AppConfig qw(:argcount);
+use AppConfig ':argcount';
 
 # internal per-variable hashes that AUTOLOAD should provide access to
 my %METHVARS;
@@ -80,7 +74,7 @@ my @GLOBAL_OK = qw( DEFAULT EXPAND VALIDATE ACTION ARGS ARGCOUNT );
 
 sub new {
     my $class = shift;
-    
+
     my $self = {
         # internal hash arrays to store variable specification information
         VARIABLE   => { },     # variable values
@@ -93,7 +87,7 @@ sub new {
         VALIDATE   => { },     # validation regexen or functions
         ACTION     => { },     # callback functions for when variable is set
         GLOBAL     => { },     # default global settings for new variables
-        
+
         # other internal data
         CREATE     => 0,       # auto-create variables when set
         CASE       => 0,       # case sensitivity flag (1 = sensitive)
@@ -103,7 +97,7 @@ sub new {
     };
 
     bless $self, $class;
-	
+
     # configure if first param is a config hash ref
     $self->_configure(shift)
         if ref($_[0]) eq 'HASH';
@@ -178,68 +172,68 @@ sub define {
         # examine each variable configuration parameter
         while (($opt, $val) = each %$cfg) {
             $opt = uc $opt;
-            
+
             # DEFAULT, VALIDATE, EXPAND, ARGS and ARGCOUNT are stored as 
             # they are;
             $opt =~ /^DEFAULT|VALIDATE|EXPAND|ARGS|ARGCOUNT$/ && do {
                 $self->{ $opt }->{ $var } = $val;
                 next;
             };
-            
+
             # CMDARG has been deprecated
             $opt eq 'CMDARG' && do {
                 $self->_error("CMDARG has been deprecated.  "
                               . "Please use an ALIAS if required.");
                 next;
             };
-            
+
             # ACTION should be a code ref
             $opt eq 'ACTION' && do {
                 unless (ref($val) eq 'CODE') {
                     $self->_error("'$opt' value is not a code reference");
                     next;
                 };
-                
+
                 # store code ref, forcing keyword to upper case
                 $self->{ ACTION }->{ $var } = $val;
-                
+
                 next;
             };
-            
+
             # ALIAS creates alias links to the variable name
             $opt eq 'ALIAS' && do {
-                
+
                 # coerce $val to an array if not already so
                 $val = [ split(/\|/, $val) ]
                     unless ref($val) eq 'ARRAY';
-                
+
                 # fold to lower case unless CASE sensitivity set
                 unless ($self->{ CASE }) {
                     @$val = map { lc } @$val;
                 }
-                
+
                 # store list of aliases...
                 $self->{ ALIASES }->{ $var } = $val;
-                
+
                 # ...and create ALIAS => VARIABLE lookup hash entries
                 foreach my $a (@$val) {
                     $self->{ ALIAS }->{ $a } = $var;
                 }
-                
+
                 next;
             };
-            
+
             # default 
             $self->_error("$opt is not a valid configuration item");
         }
-        
-    	# set variable to default value
+
+        # set variable to default value
         $self->_default($var);
-        
+
         # DEBUG: dump new variable definition
         if ($DEBUG) {
             print STDERR "Variable defined:\n";
-    	    $self->_dump_var($var);
+            $self->_dump_var($var);
         }
     }
 }
@@ -271,10 +265,10 @@ sub get {
 
     # DEBUG
     print STDERR "$self->get($variable) => ", 
-	   defined $self->{ VARIABLE }->{ $variable }
-		  ? $self->{ VARIABLE }->{ $variable }
-		  : "<undef>",
-	  "\n"
+           defined $self->{ VARIABLE }->{ $variable }
+                  ? $self->{ VARIABLE }->{ $variable }
+                  : "<undef>",
+          "\n"
           if $DEBUG;
 
     # return variable value, possibly negated if the name was "no<var>"
@@ -317,7 +311,7 @@ sub set {
         if (defined $create
             && ($create eq '1' || $variable =~ /$create/)) {
             $self->define($variable);
-            
+
             print STDERR "Auto-created $variable\n" if $DEBUG;
         }
         else {
@@ -325,13 +319,13 @@ sub set {
             return 0;
         }
     }
-    
+
     # call the validate($variable, $value) method to perform any validation
     unless ($self->_validate($variable, $value)) {
         $self->_error("$variable: invalid value: $value");
         return 0;
     }
-    
+
     # DEBUG
     print STDERR "$self->set($variable, ", 
     defined $value
@@ -339,7 +333,7 @@ sub set {
         : "<undef>",
         ")\n"
         if $DEBUG;
-    
+
 
     # set the variable value depending on its ARGCOUNT
     my $argcount = $self->{ ARGCOUNT }->{ $variable };
@@ -364,7 +358,7 @@ sub set {
 
     # call any ACTION function bound to this variable
     return &{ $self->{ ACTION }->{ $variable } }($self, $variable, $value)
-    	if (exists($self->{ ACTION }->{ $variable }));
+        if (exists($self->{ ACTION }->{ $variable }));
 
     # ...or just return 1 (ok)
     return 1;
@@ -406,7 +400,7 @@ sub varlist {
     return %set;
 }
 
-    
+
 #------------------------------------------------------------------------
 # AUTOLOAD
 #
@@ -442,7 +436,7 @@ sub AUTOLOAD {
     # a leading underscore_
     if (($attrib = $variable) =~ s/_//g) {
         $attrib = uc $attrib;
-	
+
         if (exists $METHFLAGS{ $attrib }) {
             return $self->{ $attrib };
         }
@@ -451,17 +445,17 @@ sub AUTOLOAD {
             # next parameter should be variable name
             $variable = shift;
             $variable = $self->_varname($variable);
-            
+
             # check we've got a valid variable
-#	    $self->_error("$variable: no such variable or method"), 
-#		    return undef
-#		unless exists($self->{ VARIABLE }->{ $variable });
-            
+#           $self->_error("$variable: no such variable or method"), 
+#                   return undef
+#               unless exists($self->{ VARIABLE }->{ $variable });
+
             # return attribute
             return $self->{ $attrib }->{ $variable };
         }
     }
-    
+
     # set a new value if a parameter was supplied or return the old one
     return defined($_[0])
            ? $self->set($variable, shift)
@@ -503,14 +497,13 @@ sub _configure {
 
                 # continue if the attribute is ok to be GLOBAL 
                 next if ($global =~ /(^$global_ok$)/io);
-                         
+
                 $self->_error( "\U$global\E parameter cannot be GLOBAL");
             }
             $self->{ GLOBAL } = $cfg->{ $opt };
             next;
         };
-            
-	
+
         # CASE, CREATE and PEDANTIC are stored as they are
         $opt =~ /^CASE|CREATE|PEDANTIC$/i && do {
             $self->{ uc $opt } = $cfg->{ $opt };
@@ -528,7 +521,7 @@ sub _configure {
             $self->_debug($cfg->{ $opt });
             next;
         };
-            
+
         # warn about invalid options
         $self->_error("\U$opt\E is not a valid configuration option");
     }
@@ -566,7 +559,7 @@ sub _varname {
 
     # get the actual name if this is an alias
     $variable = $self->{ ALIAS }->{ $variable }
-	if (exists($self->{ ALIAS }->{ $variable }));
+        if (exists($self->{ ALIAS }->{ $variable }));
 
     # if the variable doesn't exist, we can try to chop off a leading 
     # "no" and see if the remainder matches an ARGCOUNT_ZERO variable
@@ -584,7 +577,7 @@ sub _varname {
             }
         }
     }
-    
+
     # return the variable name
     $variable;
 }
@@ -610,16 +603,16 @@ sub _default {
         # hash array, depending on its ARGCOUNT value
         my $argcount = $self->{ ARGCOUNT }->{ $variable };
         $argcount = AppConfig::ARGCOUNT_ONE unless defined $argcount;
-        
+
         if ($argcount == AppConfig::ARGCOUNT_NONE) {
             return $self->{ VARIABLE }->{ $variable } 
-		    = $self->{ DEFAULT }->{ $variable } || 0;
+                 = $self->{ DEFAULT }->{ $variable } || 0;
         }
         elsif ($argcount == AppConfig::ARGCOUNT_LIST) {
             my $deflist = $self->{ DEFAULT }->{ $variable };
             return $self->{ VARIABLE }->{ $variable } = 
                 [ ref $deflist eq 'ARRAY' ? @$deflist : ( ) ];
-            
+
         }
         elsif ($argcount == AppConfig::ARGCOUNT_HASH) {
             my $defhash = $self->{ DEFAULT }->{ $variable };
@@ -628,7 +621,7 @@ sub _default {
         }
         else {
             return $self->{ VARIABLE }->{ $variable } 
-		    = $self->{ DEFAULT }->{ $variable };
+                 = $self->{ DEFAULT }->{ $variable };
         }
     }
     else {
@@ -688,8 +681,8 @@ sub _validate {
 
     # CODE ref
     ref($validator) eq 'CODE' && do {
-    	# run the validation function and return the result
-       	return &$validator($variable, $value);
+        # run the validation function and return the result
+        return &$validator($variable, $value);
     };
 
     # non-ref (i.e. scalar)
@@ -697,7 +690,7 @@ sub _validate {
         # not a ref - assume it's a regex
         return $value =~ /$validator/;
     };
-    
+
     # validation failed
     return 0;
 }
@@ -755,7 +748,7 @@ sub _ehandler {
         # check this is a code reference
         if (ref($handler) eq 'CODE') {
             $self->{ EHANDLER } = $handler;
-            
+
             # DEBUG
             print STDERR "installed new ERROR handler: $handler\n" if $DEBUG;
         }
@@ -763,7 +756,7 @@ sub _ehandler {
             $self->_error("ERROR handler parameter is not a code ref");
         }
     }
-   
+
     return $previous;
 }
 
@@ -811,31 +804,31 @@ sub _dump_var {
     # $var may be an alias, so we resolve the real variable name
     my $real = $self->_varname($var);
     if ($var eq $real) {
-    	print STDERR "$var\n";
+        print STDERR "$var\n";
     }
     else {
-    	print STDERR "$real  ('$var' is an alias)\n";
+        print STDERR "$real  ('$var' is an alias)\n";
         $var = $real;
     }
 
     # for some bizarre reason, the variable VALUE is stored in VARIABLE
     # (it made sense at some point in time)
     printf STDERR "    VALUE        => %s\n", 
-		defined($self->{ VARIABLE }->{ $var }) 
-		    ? $self->{ VARIABLE }->{ $var } 
-		    : "<undef>";
+                defined($self->{ VARIABLE }->{ $var }) 
+                    ? $self->{ VARIABLE }->{ $var } 
+                    : "<undef>";
 
     # the rest of the values can be read straight out of their hashes
     foreach my $param (qw( DEFAULT ARGCOUNT VALIDATE ACTION EXPAND )) {
-	printf STDERR "    %-12s => %s\n", $param, 
-		defined($self->{ $param }->{ $var }) 
-		    ? $self->{ $param }->{ $var } 
-		    : "<undef>";
+        printf STDERR "    %-12s => %s\n", $param, 
+                defined($self->{ $param }->{ $var }) 
+                    ? $self->{ $param }->{ $var } 
+                    : "<undef>";
     }
 
     # summarise all known aliases for this variable
     print STDERR "    ALIASES      => ", 
-	    join(", ", @{ $self->{ ALIASES }->{ $var } }), "\n"
+            join(", ", @{ $self->{ ALIASES }->{ $var } }), "\n"
             if defined $self->{ ALIASES }->{ $var };
 } 
 
@@ -852,14 +845,14 @@ sub _dump {
 
     print STDERR "=" x 71, "\n";
     print STDERR 
-	"Status of AppConfig::State (version $VERSION) object:\n\t$self\n";
+        "Status of AppConfig::State (version $VERSION) object:\n\t$self\n";
+
 
-    
     print STDERR "- " x 36, "\nINTERNAL STATE:\n";
     foreach (qw( CREATE CASE PEDANTIC EHANDLER ERROR )) {
         printf STDERR "    %-12s => %s\n", $_, 
-		defined($self->{ $_ }) ? $self->{ $_ } : "<undef>";
-    }	    
+                defined($self->{ $_ }) ? $self->{ $_ } : "<undef>";
+    }       
 
     print STDERR "- " x 36, "\nVARIABLES:\n";
     foreach $var (keys %{ $self->{ VARIABLE } }) {
@@ -895,7 +888,7 @@ AppConfig::State - application configuration state
 
     $state->set("foo", 123);          # trivial set/get examples
     $state->get("foo");      
-    
+
     $state->foo();                    # shortcut variable access 
     $state->foo(456);                 # shortcut variable update 
 
@@ -919,12 +912,12 @@ appear in your Perl script:
 
 The AppConfig::State module is loaded automatically by the new()
 constructor of the AppConfig module.
-      
+
 AppConfig::State is implemented using object-oriented methods.  A 
 new AppConfig::State object is created and initialised using the 
 new() method.  This returns a reference to a new AppConfig::State 
 object.
-       
+
     my $state = AppConfig::State->new();
 
 This will create a reference to a new AppConfig::State with all 
@@ -933,8 +926,8 @@ the object by passing a reference to a hash array containing
 configuration options:
 
     $state = AppConfig::State->new( {
-	CASE      => 1,
-	ERROR     => \&my_error,
+        CASE      => 1,
+        ERROR     => \&my_error,
     } );
 
 The new() constructor of the AppConfig module automatically passes all 
@@ -968,7 +961,7 @@ variable is created.  This can be used to specify configuration file
 blocks in which variables should be created, for example:
 
     $state = AppConfig::State->new( {
-	CREATE => '^define_',
+        CREATE => '^define_',
     } );
 
 In a config file:
@@ -1028,11 +1021,11 @@ The GLOBAL option allows default values to be set for the DEFAULT, ARGCOUNT,
 EXPAND, VALIDATE and ACTION options for any subsequently defined variables.
 
     $state = AppConfig::State->new({
-	GLOBAL => {
-	    DEFAULT  => '<undef>',     # default value for new vars
-	    ARGCOUNT => 1,             # vars expect an argument
-	    ACTION   => \&my_set_var,  # callback when vars get set
-	}
+        GLOBAL => {
+            DEFAULT  => '<undef>',     # default value for new vars
+            ARGCOUNT => 1,             # vars expect an argument
+            ACTION   => \&my_set_var,  # callback when vars get set
+        }
     });
 
 Any attributes specified explicitly when a variable is defined will
@@ -1054,9 +1047,9 @@ reference to a hash array may also be passed to specify configuration
 information for the variable:
 
     $state->define("foo", {
-	    DEFAULT   => 99,
-	    ALIAS     => 'metavar1',
-	});
+            DEFAULT   => 99,
+            ALIAS     => 'metavar1',
+        });
 
 Any variable-wide GLOBAL values passed to the new() constructor in the 
 configuration hash will also be applied.  Values explicitly specified 
@@ -1072,8 +1065,8 @@ The following configuration options may be specified
 The DEFAULT value is used to initialise the variable.  
 
     $state->define("drink", {
-	    DEFAULT => 'coffee',
-	});
+            DEFAULT => 'coffee',
+        });
 
     print $state->drink();        # prints "coffee"
 
@@ -1084,17 +1077,20 @@ this variable.  A single alias should be specified as a string.  Multiple
 aliases can be specified as a reference to an array of alternatives or as 
 a string of names separated by vertical bars, '|'.  e.g.:
 
+    # either
     $state->define("name", {
-	    ALIAS  => 'person',
-	});
-or
+            ALIAS  => 'person',
+        });
+
+    # or
     $state->define("name", {
-	    ALIAS => [ 'person', 'user', 'uid' ],
-	});
-or
+            ALIAS => [ 'person', 'user', 'uid' ],
+        });
+
+    # or
     $state->define("name", {
-	    ALIAS => 'person|user|uid',
-	});
+            ALIAS => 'person|user|uid',
+        });
 
     $state->user('abw');     # equivalent to $state->name('abw');
 
@@ -1155,8 +1151,8 @@ See those two modules for more information on the format and meaning of
 these options.
 
     $state->define("name", {
-	    ARGS => "=i@",
-	});
+            ARGS => "=i@",
+        });
 
 =item EXPAND 
 
@@ -1194,6 +1190,9 @@ to the values of the relevant AppConfig::State variables.
 
 Indicates that '~' or '~uid' patterns in the string should be 
 expanded to the current users ($<), or specified user's home directory.
+In the first case, C<~> is expanded to the value of the C<HOME>
+environment variable.  In the second case, the C<getpwnam()> method
+is used if it is available on your system (which it isn't on Win32).
 
 =item EXPAND_ENV
 
@@ -1233,12 +1232,12 @@ above) then this value will be used as the default VALIDATE for each
 variable unless otherwise specified.
 
     $state->define("age", {
-    	    VALIDATE => '\d+',
-	});
+            VALIDATE => '\d+',
+        });
 
     $state->define("pin", {
-	    VALIDATE => \&check_pin,
-	});
+            VALIDATE => \&check_pin,
+        });
 
 =item ACTION
 
@@ -1256,11 +1255,11 @@ Example:
     $state->define("foo", { ACTION => \&my_notify });
 
     sub my_notify {
-	my $state = shift;
-	my $var   = shift;
-	my $val   = shift;
+        my $state = shift;
+        my $var   = shift;
+        my $val   = shift;
 
-	print "$variable set to $value";
+        print "$variable set to $value";
     }
 
     $state->foo(42);        # prints "foo set to 42"
@@ -1269,8 +1268,6 @@ Be aware that calling C<$state-E<gt>set()> to update the same variable
 from within the ACTION function will cause a recursive loop as the
 ACTION function is repeatedly called.  
 
-=item 
-
 =back
 
 =head2 DEFINING VARIABLES USING THE COMPACT FORMAT
@@ -1286,9 +1283,9 @@ after the variable name(s) and/or aliases.
 The following examples are equivalent:
 
     $state->define("foo", { 
-	    ALIAS => [ 'bar', 'baz' ],
-	    ARGS  => '=i',
-	});
+            ALIAS => [ 'bar', 'baz' ],
+            ARGS  => '=i',
+        });
 
     $state->define("foo|bar|baz=i");
 
@@ -1395,13 +1392,9 @@ debugging purposes.
 
 Andy Wardley, E<lt>abw@wardley.orgE<gt>
 
-=head1 REVISION
-
-$Revision: 1.61 $
-
 =head1 COPYRIGHT
 
-Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
 
 Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 
